%
%   Step_3_create_monthly_GDP_using_METI_all_index.m
%
%  This is step 3 of 3 of the creation of working file from raw data.
%
%  input data: "raw quarterly and monthly output.xls", "temporary_data.mat"
%  output file: "working file.xls"
%  
%  user-defined function: "allocate_between_months_f.m", "merge_fh.m", "HPfilter.m"
%
%
%  February 2014
%  
%
%%

clear;clc;

disp('********************** program starts ************************');

% import Matlab .mat data: the only purpose is to retrieve "yr_month"
load temporary_data;
whos -file temporary_data
yr_month_all=yr_month;


% Read from "quarterly and monthly series.xls"
infile='raw quarterly and monthly output';


% write to "monthly data.xls"
outfile='working file.xls';


% METI all industry index, SA
yr_month=xlsread(infile,'all industry index','a7:a134');
        % year-month, from Jan. 2003 to August 2013
all_index=xlsread(infile,'all industry index','p7:p134');
        % METI all industry index from Januaury 2003 to August 2013

% old index
all_index_old=xlsread(infile,'all industry index, old','t70:t249');
        % METI all industry index from January 1988 to December 2002
yr_month_old=[];
for year=1988:2002;
    for month=1:12;
        yr_month_old=cat(1,yr_month_old,year*100+month);
    end;
end;

% concatenate all-industry index over its old index
yr_month=cat(1,yr_month_old,yr_month);
all_index=cat(1,all_index_old,all_index);
z=[yr_month all_index];
% z has
% column 1: year and month (e.g., 201209)
% column 2: METI all industry index
temp=merge_fh(yr_month_all,z);
all_index=temp(:,4);
clear temp z all_index_old yr_month_old;

        


% read quarterly GDP
% define overall year-quarter
yr_quarter_all=[];
for year=1980:2013;
    for quarter=1:4;
        yr_quarter_all=cat(1,yr_quarter_all,year*100+quarter);
    end;
end;

% read chained seasonally adjusted GDP, benchmark year = 2000
quarterly_GDP_old=xlsread(infile,'Cabinet Office 1','b8:b131');
            % from 1980:Q1 - 2010:Q4
yr_quarter=[];
for year=1980:2010;
    for quarter=1:4;
        yr_quarter=cat(1,yr_quarter,year*100+quarter);
    end;
end;
z=[yr_quarter quarterly_GDP_old];
% z has
% column 1: year and quarter (e.g., 200301)
% column 2: real GDP, benchmark = 2000
temp=merge_fh(yr_quarter_all,z);
yr_quarter_all=temp(:,1);
quarterly_GDP_old=temp(:,4);
clear temp z yr_quarter;

% read current GDP series
quarterly_GDP=xlsread(infile,'Cabinet Office 2','b8:b87');
    % Cabinet Office, quarterly GDP, SA, base=2005, 1994:Q1- 2013:Q4
yr_quarter=[];
for year=1994:2013;
    for quarter=1:4;
        yr_quarter=cat(1,yr_quarter,year*100+quarter);
    end;
end;
z=[yr_quarter quarterly_GDP];
% z has
% column 1: year and quarter (e.g., 200301)
% column 2: real GDP, benchmark = 2005
temp=merge_fh(yr_quarter_all,z);
yr_quarter_all=temp(:,1);
quarterly_GDP=temp(:,4);
clear temp z yr_quarter;

% combine old and current GDP series
j=find(yr_quarter_all==199401);
ratio=quarterly_GDP(j)/quarterly_GDP_old(j);
temp=quarterly_GDP_old(1:j)*ratio;
quarterly_GDP=cat(1,temp,quarterly_GDP(j+1:end));
clear quarterly_GDP_old



% allocate qurterly GDP between the three months
quarter_all=mod(yr_quarter_all,10);
year_all=(yr_quarter_all-quarter_all)/100;
z=[];
for q=1:numel(quarterly_GDP);
    total_Y=quarterly_GDP(q);
    % identify the 3 months forming the quarter
    if quarter_all(q)==1;
        m_a=01;
        m_b=02;
        m_c=03;
    elseif quarter_all(q)==2;
        m_a=04;
        m_b=05;
        m_c=06;
    elseif quarter_all(q)==3;
        m_a=07;
        m_b=08;
        m_c=09;
    elseif quarter_all(q)==4;
        m_a=10;
        m_b=11;
        m_c=12;
    end;
    yr_month_a=year_all(q)*100+m_a;
    yr_month_b=year_all(q)*100+m_b;
    yr_month_c=year_all(q)*100+m_c;
    % pull out all industry index
    y_a=all_index(yr_month_all==yr_month_a);
    y_b=all_index(yr_month_all==yr_month_b);
    y_c=all_index(yr_month_all==yr_month_c);
    % imputed growth rate
    g=NaN(2,1);
    g(1)=log(y_b)-log(y_a);
    g(2)=log(y_c)-log(y_b);
    % now do the imputation
    if ~isnan(g(1)) && ~isnan(g(2));
        y0=ones(3,1)*log(total_Y);
        temp=fsolve(@(y) allocate_between_months_f(y,g,total_Y),y0);
        v=exp(temp);
        % check the solution
        temp=[total_Y-ones(1,3)*v/3;...
            log(v(2))-log(v(1))-g(1);
            log(v(3))-log(v(2))-g(2)];
        if max(temp)>1e-8;
            disp('  Not really a solution!!!');
            pause;
        end;
    else;
        v=NaN*ones(3,1);
    end;
    
    % record the allocatted monthly GDPs
    temp=[yr_month_a;yr_month_b;yr_month_c];
    z=cat(1,z,[temp v]);
end;
% z has
% column 1: year-month (e.g., 198001)
% column 2: interpolated monthly GDP
temp=merge_fh(yr_month_all,z);
yr_month_all=temp(:,1);
interpolated_monthly_GDP=temp(:,4);
clear temp z;




% calculate monthly potential GDP
% obtain quarterly GDP gap
GDP_gap=xlsread(infile,'Cabinet Office 3','d2:d135');
    % Cabinet Office, GDP gap, 1980:Q1- 2013:Q2
temp=[];
for year=1980:2013;
    for quarter=1:4;
        temp=cat(1,temp,year*100+quarter);
    end;
end;
temp=temp(1:end-2);  % drop 2013:Q3 and Q4
z=[temp GDP_gap];
% z has
% column 1: year and quarter (e.g., 200301)
% column 2: GDP gap
temp=merge_fh(yr_quarter_all,z);
yr_quarter_all=temp(:,1);
GDP_gap=temp(:,4);
clear temp z;

% calculate implied quarterly potential GDP
quarterly_potential_GDP=NaN(numel(yr_quarter_all),1);
for q=1:numel(yr_quarter_all);
    quarterly_potential_GDP(q)=quarterly_GDP(q)/(1+GDP_gap(q)/100);
end;



% HP filter potential GDP?
in_quarters=(~isnan(quarterly_potential_GDP) & yr_quarter_all<=201212);
temp=quarterly_potential_GDP(in_quarters);
temp=HPfilter(log(temp),1600);
j=find(in_quarters==1);
j_start=j(1);
j_end=j(end);
quarterly_potential_GDP=NaN*quarterly_potential_GDP;
quarterly_potential_GDP(j_start:j_end)=exp(temp);




% allocate qurterly potential GDP between the three months
quarter_all=mod(yr_quarter_all,10);
year_all=(yr_quarter_all-quarter_all)/100;
z=[];
for q=1:numel(quarterly_potential_GDP);
    % identify the 3 months forming the quarter
    if quarter_all(q)==1;
        m_a=01;
        m_b=02;
        m_c=03;
    elseif quarter_all(q)==2;
        m_a=04;
        m_b=05;
        m_c=06;
    elseif quarter_all(q)==3;
        m_a=07;
        m_b=08;
        m_c=09;
    elseif quarter_all(q)==4;
        m_a=10;
        m_b=11;
        m_c=12;
    end;
    yr_month_a=year_all(q)*100+m_a;
    yr_month_b=year_all(q)*100+m_b;
    yr_month_c=year_all(q)*100+m_c;

    if q==1;
        v=ones(3,1)*quarterly_potential_GDP(q);
                % for now, allocate equally for the first quarterly observation
    else;
        total_Y=quarterly_potential_GDP(q);
        % imputed growth rate
        g=NaN(2,1);
        temp=log(quarterly_potential_GDP(q))-log(quarterly_potential_GDP(q-1));
        temp=temp/3;
        g(1)=temp;
        g(2)=temp;
        % now do the interpolation
        if ~isnan(g(1)) && ~isnan(g(2));
            y0=ones(3,1)*log(total_Y);
            temp=fsolve(@(y) allocate_between_months_f(y,g,total_Y),y0);
            v=exp(temp);
        else;
            v=NaN*ones(3,1);
        end;
    end;
    
    % record the allocated monthly potential GDPs
    temp=[yr_month_a;yr_month_b;yr_month_c];
    if q==1;
        z=[temp v];
    elseif q>1;
        z=cat(1,z,[temp v]);
    end;
    
end;
% z has
% column 1: year-month (e.g., 198001)
% column 2: interpolated monthly potential GDP
temp=merge_fh(yr_month_all,z);
interpolated_monthly_potential_GDP=temp(:,4);
clear temp z;



% obtain IP (Index of Industrial Production, sasonally adjusted)
z=xlsread(infile,'IP from Datastream','a2:b518');
% z has
% column 1: year-month (e.g., 200708)
% column 2: IP (source: Datastream)
temp=merge_fh(yr_month_all,z);
IP=temp(:,4);
clear temp z;




% write to Excel file
yr_month=yr_month_all;
header={...
    'yr_month',...
    'r_bar','r_ave',...
    'M','M_req',...
    'core_core_CPI_SA','core_CPI_NSA_ctax',...
    'core_CPI_NSA','core_core_CPI_NSA',...
    'interpolated_monthly_GDP','interpolated_monthly_potential_GDP',...
    'IP','base'...
    };
xlswrite(outfile,header,'Sheet1','a1');
z=[...
    yr_month,...
    r_bar,r_ave,...
    M,M_req,...
    core_core_CPI_SA,core_CPI_NSA_ctax,...
    core_CPI_NSA,core_core_CPI_NSA,...
    interpolated_monthly_GDP,interpolated_monthly_potential_GDP,...
    IP,base...
    ];
xlswrite(outfile,z,'Sheet1','a2');
